package memento;

import java.io.File;
import java.util.Iterator;
import org.junit.Assert;
import org.junit.Test;

/**
 *
 * @author thsc
 */
public class WishlistTests {
    private static final String GOLD = "Gold";
    private static final String EDELSTEINE = "Edelsteine";
    private static final String PROG2_MARK_1 = "eine 1 in Programmieren 2";
    private static final String SCHOKOLADE = "Schokolode";
    
    private Wishlist currentList = null;
    
    private static Wishlist getWishlist() {
        return new WishlistImpl();
    }
    
    public void someGoodTests() {
        Wishlist wishes = this.currentList;
        Assert.assertNotNull("wish list must no be empty", wishes);

        Assert.assertEquals(3, wishes.size());
         
        boolean goldFound = false;
        boolean edelsteineFound = false;
        boolean prog2Found = false;
        
        Iterator<String> wishIter = wishes.getWishes();
        while(wishIter.hasNext()) {
            String wish = wishIter.next();
            if(wish.equalsIgnoreCase(EDELSTEINE)) {
                edelsteineFound = true;
            } else if(wish.equalsIgnoreCase(PROG2_MARK_1)) {
                prog2Found = true;
            } else if(wish.equalsIgnoreCase(GOLD)) {
                goldFound = true;
            }
        }
        
        wishes.print(System.out);
        
        Assert.assertTrue(edelsteineFound);
        Assert.assertTrue(prog2Found);
        Assert.assertTrue(goldFound);
        
    }
    
    private void addWishes() {
        Wishlist wishes = this.currentList;

        wishes.addWish(GOLD);
        wishes.addWish(GOLD); // twice!
        wishes.addWish(EDELSTEINE);
        wishes.addWish(PROG2_MARK_1);

        Assert.assertEquals(4, wishes.size());

        // nothing should happen
        wishes.removeWish(SCHOKOLADE);

        wishes.removeWish(GOLD);
    }
    
    @Test
    public void fromScratch() {
        this.currentList = WishlistTests.getWishlist();
        this.addWishes();
        this.someGoodTests();
    }
    
    @Test
    public void fromScratchToFileAndBack() {
        this.currentList = WishlistTests.getWishlist();
        this.addWishes();
        this.someGoodTests();
        
        // create memento
        File file = new File("wishListFile.txt");
        FileWishlistMemento fileMemento = new FileWishlistMemento(file);
        // save
        this.currentList.saveToMemento(fileMemento);
        
        // restore in fresh list
        this.currentList = new WishlistImpl();
        this.currentList.restoreFromMemento(fileMemento);
        
        // run same tests
        this.someGoodTests();
    }
}
